using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Reactive;
using System.Text.RegularExpressions;
using DynamicData;
using Microsoft.EntityFrameworkCore;
using MsBox.Avalonia;
using Prakt12.DBModels;
using ReactiveUI;

namespace Prakt12.ViewModels
{
	public class AddTeacherUCViewModel : ViewModelBase
	{
        public AddTeacherUCViewModel()
        {
            AddNewTeacherCommand = ReactiveCommand.Create<string>(AddNewTeacherCommandRun);
            CancelAddNewTeacherCommand = ReactiveCommand.Create<string>(CancelAddNewTeacherCommandRun);
            LoadDataFromBaseAsync();
        }


        public event Action AddNewTeacherPageWasClosed;


        public string Patronymic
		{
			get => _patronymic;
			set => this.RaiseAndSetIfChanged(ref _patronymic, value);
		}
        public string Name
        {
            get => _name;
            set => this.RaiseAndSetIfChanged(ref _name, value);
        }
        public string Surname
        {
            get => _surname;
            set => this.RaiseAndSetIfChanged(ref _surname, value);
        }
        public string Email
        {
            get => _email;
            set => this.RaiseAndSetIfChanged(ref _email, value);
        }
        public string Phone
        {
            get => _phone;
            set => this.RaiseAndSetIfChanged(ref _phone, value);
        }

        public DateTimeOffset Date
        {
            get => _date;
            set => this.RaiseAndSetIfChanged(ref _date, value);
        }

        public int GenderComboboxId
        {
            get => _genderComboboxId;
            set => this.RaiseAndSetIfChanged(ref _genderComboboxId, value);
        }
        public int TeacherWorkTime
        {
            get => _teacherWorkTime;
            set => this.RaiseAndSetIfChanged(ref _teacherWorkTime, value);
        }
        public int SelectedComboBoxSubjectId
        {
            get => _selectedComboBoxSubjectId;
            set
            {
                this.RaiseAndSetIfChanged(ref _selectedComboBoxSubjectId, value);
                if(value != -1)
                {
                    _selectedSubjects.Add(UnselectedSubjects[value]);
                    _unselectedSubjects.RemoveAt(value);
                    SelectedComboBoxSubjectId = -1;
                }
            }
        }
        public int SelectedListBoxSubjectId
        {
            get => _selectedListBoxSubjectId;
            set
            {
                this.RaiseAndSetIfChanged(ref _selectedListBoxSubjectId, value);
                if(value != -1)
                {
                    _unselectedSubjects.Add(_selectedSubjects[value]);
                    _selectedSubjects.RemoveAt(value);
                    SelectedListBoxSubjectId = -1;
                }
            }
        }

        public ObservableCollection<Subject> UnselectedSubjects
        {
            get => _unselectedSubjects;
            set => this.RaiseAndSetIfChanged(ref _unselectedSubjects, value);
        }
        public ObservableCollection<Subject> SelectedSubjects
        {
            get => _selectedSubjects;
            set => this.RaiseAndSetIfChanged(ref _selectedSubjects, value);
        }

        public ReactiveCommand<string, Unit> AddNewTeacherCommand { get; }
        public ReactiveCommand<string, Unit> CancelAddNewTeacherCommand { get; }


        private string _patronymic = "";
		private string _name = "";
		private string _surname = "";
        private string _email = "";
        private string _phone = "";

        private DateTimeOffset _date = DateTimeOffset.Now;

        private int _genderComboboxId = 0;
        private int _teacherWorkTime = 0;
        private int _selectedComboBoxSubjectId = -1;
        private int _selectedListBoxSubjectId = -1;

        private ObservableCollection<Subject> _unselectedSubjects = new ObservableCollection<Subject>();
        private ObservableCollection<Subject> _selectedSubjects = new ObservableCollection<Subject>();


        private async void LoadDataFromBaseAsync()
        {
            using (_43pVorobiewContext DB = new _43pVorobiewContext())
            {
                UnselectedSubjects = new ObservableCollection<Subject>(await DB.Subjects.ToListAsync());
            }
        }

        private void AddNewTeacherCommandRun(string param)
        {
            if(Surname.Length == 0 || Name.Length == 0 || Patronymic.Length == 0)
            {
                MessageBoxManager.GetMessageBoxStandard("Warning", "  !", MsBox.Avalonia.Enums.ButtonEnum.Ok).ShowAsync();
                return;
            }
            if(Date > DateTimeOffset.Now)
            {
                MessageBoxManager.GetMessageBoxStandard("Warning", "  !", MsBox.Avalonia.Enums.ButtonEnum.Ok).ShowAsync();
                return;
            }
            if (!Regex.IsMatch(Email, "^[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?$"))
            {
                MessageBoxManager.GetMessageBoxStandard("Warning", "Email !", MsBox.Avalonia.Enums.ButtonEnum.Ok).ShowAsync();
                return;
            }
            if(SelectedSubjects.Count == 0)
            {
                MessageBoxManager.GetMessageBoxStandard("Warning", "    !", MsBox.Avalonia.Enums.ButtonEnum.Ok).ShowAsync();
                return;
            }

            using (_43pVorobiewContext DB = new())
            {
                List<TeachersSubject> list = new();
                foreach(var item in SelectedSubjects)
                {
                    list.Add(new TeachersSubject() 
                    {
                        IdSubject = item.IdSubjects
                    });
                }
                DB.Teachers.Add(new Teacher()
                {
                    TeacherSurname = Surname,
                    TeacherName = Name,
                    TeacherEmail = Email,
                    TeacherPatronymic = Patronymic,
                    TeacherPhoneNumber = Phone,
                    IdGender = GenderComboboxId + 1,
                    TeacherWorkTime = TeacherWorkTime,
                    TeacherBirthdate = DateOnly.FromDateTime((Date.Date)),
                    TeachersSubjects = list
                });
                DB.SaveChanges();
                MessageBoxManager.GetMessageBoxStandard("Success", "  !", MsBox.Avalonia.Enums.ButtonEnum.Ok).ShowAsync();
            }
        }

        private void CancelAddNewTeacherCommandRun(string param) => AddNewTeacherPageWasClosed?.Invoke();
    }
}